// simple.frag

#version 120
#extension GL_EXT_gpu_shader3 : enable

uniform sampler2D textureImage;
uniform int textureMode;
uniform int fogMode;

varying float diffuseTotal;
varying float specularTotal;

void FogEffect(){

    float z = gl_FragCoord.z/ gl_FragCoord.w;
	
    vec4 fogColor    = gl_Fog.color;
    fogColor[3] = gl_FragColor[3];
    float fogDensity = gl_Fog.density;

    float fogFactor = 0.0;
    if (fogMode == 3){
	fogFactor = (gl_Fog.end - z) / (gl_Fog.end - gl_Fog.start);
    }
    else if (fogMode == 2){
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * z * LOG2 );
    }
    else {
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * gl_Fog.density * z * z * LOG2 );
    }
    
    fogFactor = clamp(fogFactor, 0.0, 1.0);
 
    gl_FragColor = mix(fogColor, gl_FragColor, fogFactor);
}

void main (void)
{
    vec4 texColor = vec4(1.0,1.0,1.0,1.0); 
    if (textureMode == 1)
	texColor = texture2D(textureImage, gl_TexCoord[0].st);

    float alpha = gl_FrontLightProduct[0].ambient[3];
    if (alpha > 1.0) alpha = 1.0;
    
    gl_FragColor = gl_FrontLightProduct[0].ambient + 
	gl_FrontMaterial.diffuse * diffuseTotal +
	gl_FrontMaterial.specular * specularTotal;
    gl_FragColor *= texColor;
    gl_FragColor[3] = alpha;

    if (fogMode != 0) FogEffect();
}
