// simple.frag

#version 130
#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D textureImage;
uniform int fogMode;
uniform int dissolveMode;
uniform int lightNumber;
uniform float clipThreshold;

in vec3 position;
in vec3 normal;
in vec4 diffuseAndAmbient;
//in vec4 ambient0;

void FogEffect(){

    float z = gl_FragCoord.z/ gl_FragCoord.w;
	
    vec4 fogColor    = gl_Fog.color;
    fogColor[3] = gl_FragColor[3];
    float fogDensity = gl_Fog.density;

    float fogFactor = 0.0;
    if (fogMode == 3){
	fogFactor = (gl_Fog.end - z) / (gl_Fog.end - gl_Fog.start);
    }
    else if (fogMode == 2){
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * z * LOG2 );
    }
    else {
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * gl_Fog.density * z * z * LOG2 );
    }
    
    fogFactor = clamp(fogFactor, 0.0, 1.0);
 
    gl_FragColor = mix(fogColor, gl_FragColor, fogFactor);
}

void main (void)
{
    vec4 texColor = texture2D(textureImage, gl_TexCoord[0].st);
    if (texColor[3] < clipThreshold) discard;

    texColor[0] += 0.0001;
    texColor[1] += 0.0001;
    texColor[2] += 0.0001;
    
    gl_FragColor = texColor * gl_Color * diffuseAndAmbient;
    float alpha = gl_FragColor[3];

    if (dissolveMode >= 1){
	float bound = 0.015;
	if (dissolveMode == 2) bound = 1;
	
	if (alpha <= bound){
	    float crit = fract(sin(dot(position.xy,vec2(12.9898,78.233))) * 43758.5453) * bound;
	    if (gl_FragColor[3] <= crit) discard;
	    else gl_FragColor[3] = bound;
	}
    }
    
    if (fogMode != 0) FogEffect();
}
