// simple.frag

#version 120
#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D textureImage0;
uniform sampler2D textureImage1;
uniform int fogMode;
uniform int lightNumber;
uniform int lightRecieve[7];
uniform float clipThreshold;

varying vec3 position;
varying vec3 viewVec;
varying vec3 normal;
varying vec3 binormal;
varying vec3 tangent;

void FogEffect(){

    float z = gl_FragCoord.z/ gl_FragCoord.w;
	
    vec4 fogColor    = gl_Fog.color;
    fogColor[3] = gl_FragColor[3];
    float fogDensity = gl_Fog.density;

    float fogFactor = 0.0;
    if (fogMode == 3){
	fogFactor = (gl_Fog.end - z) / (gl_Fog.end - gl_Fog.start);
    }
    else if (fogMode == 2){
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * z * LOG2 );
    }
    else {
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * gl_Fog.density * z * z * LOG2 );
    }
    
    fogFactor = clamp(fogFactor, 0.0, 1.0);
 
    gl_FragColor = mix(fogColor, gl_FragColor, fogFactor);
}

void main (void)
{
    vec4 texColorBase = texture2D(textureImage1, gl_TexCoord[0].st);
    vec4 texNormal    = texture2D(textureImage0, gl_TexCoord[0].st);

    if (texColorBase[3] < 0.99) discard;

    float r = texNormal.r;
    float g = texNormal.g;
    float b = texNormal.b;

    vec3 fnormal = normalize(
	normal * (b * 2.0f - 1.0f) +
	tangent * (r * 2.0f - 1.0f) +
	binormal * (g * 2.0f - 1.0f)
	);

    float alpha = gl_FrontLightProduct[0].ambient[3];
    //if (alpha > 1.0) alpha = 1.0;

    // ׂ͂ĔFF
    float diffuseTotal = 0.0;
    float specularTotal = 0.0;

    //vec3 viewVec = -normalize(position);

    for (int i = 0; i < lightNumber; i++){
	//if (lightRecieve[i] == 0) continue;
	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	// fBt[Y 
	float diffuse = dot(lightVec, fnormal);
	
	if (diffuse > 0.0){
	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    float lightFactor = 1;//GetSpotEffect(i, spotAngleCos);
	    
	    // W
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);

	    //˃xNg 
	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = 
		pow(max(dot(viewVec, reflectVec), 0.0),
		    10.0/*gl_FrontMaterial.shininess*/) * lightFactor;
	    if (specular < 0) specular = 0;
	    if (specular > 1.0f) specular = 1.0f;
	    diffuseTotal += diffuse * gl_LightSource[i].diffuse.r * lightFactor;
	    specularTotal += specular * gl_LightSource[i].diffuse.r;
	}
    }

    gl_FragColor = (gl_FrontLightProduct[0].ambient + 
		    gl_FrontMaterial.diffuse * diffuseTotal +
		    gl_FrontMaterial.specular * specularTotal);

    gl_FragColor *= texColorBase * gl_Color;

    if (fogMode != 0) FogEffect();
    
    gl_FragColor[3] = alpha;
}
